@@ -134,6 +134,45 @@ def find_category_interactions ( istr ) :
     interacting_categories = [ [all_cats[i-1],all_cats[i]] for i in range(1,len(interacting)) if interacting[i] ]
     return ( interacting_categories )
 
+def create_encoding_data_frame ( journal_df , formula , bVerbose = False ) :
+    #
+    # THE JOURNAL_DF IS THE COARSE GRAINED DATA (THE MODEL)
+    # THE FORMULA IS THE SEMANTIC DESCRIPTION OF THE PROBLEM
+    #
+    interaction_pairs = find_category_interactions ( formula.split('~')[1] )
+    add_pairs = []
+    sjdf = set(journal_df.index)
+    if len( interaction_pairs ) > 0 :
+        for pair in interaction_pairs :
+            cpair = [ 'C('+p+')' for p in pair ]
+            upair = [ pp*(pp in sjdf)+cp*(cp in sjdf and not pp in sjdf) for (pp,cp) in zip( pair,cpair) ]
+            journal_df.loc[ ':'.join(upair) ] = [ p[0]+'-'+p[1] for p in journal_df.loc[ upair,: ].T.values ]
+            add_pairs.append(':'.join(upair))
+    use_categories = list(set(find_category_variables(formula.split('~')[1])))
+    cusecats = [ 'C('+p+')' for p in use_categories ]
+    use_categories = [ u*( u in sjdf) + cu *( cu in sjdf ) for (u,cu) in zip(use_categories,cusecats) ]
+    use_categories = [ *use_categories,*add_pairs ]
+    #
+    if len( use_categories )>0 :
+        encoding_df = create_encoding_journal ( use_categories , journal_df ).T
+    else :
+        encoding_df = None
+    #
+    if bVerbose :
+        print ( [ v for v in encoding_df.columns.values ] )
+        print ( 'ADD IN ANY LINEAR TERMS AS THEIR OWN AXIS' )
+    #
+    # THIS TURNS THE MODEL INTO A MIXED LINEAR MODEL
+    add_df = journal_df.loc[ [c.replace(' ','') for c in formula.split('~')[1].split('+') if not 'C('in c],: ]
+    if len(add_df)>0 :
+        if encoding_df is None :
+            encoding_df = add_df.T
+        else :
+            encoding_df = pd.concat([ encoding_df.T ,
+                            journal_df.loc[ [ c.replace(' ','') for c in formula.split('~')[1].split('+') if not 'C(' in c] , : ] ]).T
+    return ( encoding_df.apply(pd.to_numeric) )
+
+
 def interpret_problem ( analyte_df , journal_df , formula , bVerbose=False ) :
     #
     # THE JOURNAL_DF IS THE COARSE GRAINED DATA (THE MODEL)
@@ -171,11 +210,10 @@ def interpret_problem ( analyte_df , journal_df , formula , bVerbose=False ) :
 
 
 def calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
-			analyte_df = None , journal_df = None ,
-			bVerbose = False , synonyms = None ,
-			blur_cutoff = 99.8 , exclude_labels_from_centroids = [''] ,
-			study_axii = None , owner_by = 'tesselation' ):
-
+                                     analyte_df = None , journal_df = None ,
+                                     bVerbose = False , synonyms = None ,
+                                     blur_cutoff = 99.8 , exclude_labels_from_centroids = [''] ,
+                                     study_axii = None , owner_by = 'tesselation' ):
     if bVerbose :
         print ( np.shape(encoding_df) )
         print ( np.shape(analyte_df)  )
@@ -295,8 +333,6 @@ def run_rpls_regression ( analyte_df , journal_df , formula ,
                           bDeveloperTesting = False ,
                           study_axii = None , owner_by = 'tesselation'
                         ) :
-    inventors__ = "Richard Tjörnhammar (RT) and Edward Tjörnhammar"
-    NOTE__ = "Edward Tjörnhammar (ET) early major contributor to this method. Inventors: "+inventors__+". RT is the main developer."
 
     encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
     from sklearn.cross_decomposition import PLSRegression as PLS
@@ -311,45 +347,48 @@ def run_rpls_regression ( analyte_df , journal_df , formula ,
     scorex = rpls_res.x_scores_
 
     res_df = calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
-    			journal_df = journal_df, analyte_df = analyte_df , blur_cutoff = blur_cutoff ,
-    			bVerbose = bVerbose, exclude_labels_from_centroids = exclude_labels_from_centroids ,
-			study_axii = study_axii , owner_by = owner_by )
+                       journal_df = journal_df, analyte_df = analyte_df , blur_cutoff = blur_cutoff ,
+                       bVerbose = bVerbose, exclude_labels_from_centroids = exclude_labels_from_centroids ,
+                       study_axii = study_axii , owner_by = owner_by )
 
     return ( res_df )
 
 import impetuous.fit as ifit
 import impetuous.clustering as icluster
 def run_shape_alignment_clustering ( analyte_df , journal_df , formula, bVerbose = False ) :
-	NOTE_ = "This is just a kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
-	encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
+        NOTE_ = "This is just a kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
+        encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
 
-	Q = encoding_df.T.apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).values
-	P = analyte_df   .apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).values
+        Q = encoding_df.T.apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).values
+        P = analyte_df   .apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).values
 
-	centroids = ifit.ShapeAlignment( P, Q ,
-				bReturnTransform = False ,
-				bShiftModel = True ,
-				bUnrestricted = True )
-	#
-	# FOR DIAGNOSTIC PURPOSES
-	centroids_df = pd.DataFrame ( centroids ,
-			index = encoding_df.columns ,
-			columns = encoding_df.index )
-	lookup_ = {i:n for n,i in zip( centroids_df.index,range(len(centroids_df.index)) ) }
+        centroids = ifit.ShapeAlignment( P, Q ,
+                        bReturnTransform = False ,
+                        bShiftModel = True ,
+                        bUnrestricted = True )
+        #
+        # FOR DIAGNOSTIC PURPOSES
+        centroids_df = pd.DataFrame ( centroids ,
+                        index = encoding_df.columns ,
+                        columns = encoding_df.index )
+        lookup_ = {i:n for n,i in zip( centroids_df.index,range(len(centroids_df.index)) ) }
 
-	labels , centroids = icluster.seeded_kmeans( P , centroids )
+        labels , centroids = icluster.seeded_kmeans( P , centroids )
 
-	res_df = pd.DataFrame( [labels] , columns=analyte_df.index , index=['cluster index'] )
-	nam_df = pd.DataFrame( [ lookup_[l] for l in labels ] ,
-				 columns=['cluster name'] ,
-				 index = analyte_df.index ).T
+        res_df = pd.DataFrame( [labels] , columns=analyte_df.index , index=['cluster index'] )
+        nam_df = pd.DataFrame( [ lookup_[l] for l in labels ] ,
+                     columns = ['cluster name'] , index = analyte_df.index ).T
 
-	res_df = pd.concat( [ res_df , nam_df ] )
-	clusters_df = pd.concat( [ centroids_df, pd.DataFrame( res_df.T.groupby('cluster name').apply(len),columns=['size']) ] ,axis=1 )
+        res_df = pd.concat( [ res_df , nam_df ] )
+        clusters_df = pd.concat( [ centroids_df, pd.DataFrame( res_df.T.groupby('cluster name').apply(len),columns=['size']) ] ,axis=1 )
 
-	return ( res_df , clusters_df )
+        return ( res_df , clusters_df )
 
-def knn_clustering_alignment( P , Q ) :
+
+def kmeans_clustering_alignment ( P, Q , bHighDim = False ) :
+    return ( knn_clustering_alignment( P , Q , bHighDim = bHighDim ) )
+
+def knn_clustering_alignment( P , Q , bHighDim=False ) : # ITS A SEEDED KMEANS THE NAME KNN IS A MISNOMER
 
     NOTE_ = "This is just a standard kmeans in arbitrary dimensions that start out with centroids that have been shape aligned"
     ispanda = lambda P: 'pandas' in str(type(P)).lower()
@@ -357,7 +396,11 @@ def knn_clustering_alignment( P , Q ) :
     P_ = BustedPanda ( P )
     Q_ = BustedPanda ( Q )
 
-    centroids = ifit .ShapeAlignment ( P_ , Q_ ,
+
+    if bHighDim :
+        centroids = ifit .HighDimensionalAlignment ( P_ , Q_ )
+    else :
+        centroids = ifit .ShapeAlignment ( P_ , Q_ ,
                     bReturnTransform = False ,
                     bShiftModel      = True  ,
                     bUnrestricted    = True  )
@@ -387,9 +430,14 @@ def tol_check( val, TOL=1E-10 ):
 
 ispanda = lambda P : 'pandas' in str(type(P)).lower()
 
-def multifactor_solution ( analyte_df , journal_df , formula ) :
+def multifactor_solution ( analyte_df , journal_df , formula ,
+                           bLegacy = False ) :
     A , J , f = analyte_df , journal_df , formula
-    encoding_df = interpret_problem ( analyte_df = A , journal_df = J , formula = f ).T
+    if bLegacy :
+        encoding_df = interpret_problem ( analyte_df = A , journal_df = J , formula = f ).T
+    else :
+        encoding_df = create_encoding_data_frame ( journal_df = J , formula = f ).T
+
     solution_ =  solve ( A.T, encoding_df.T )
     tol_check ( solution_[1] )
     beta_df  = pd.DataFrame ( solution_[0] , index=A.index , columns=encoding_df.index )
@@ -468,6 +516,33 @@ def multivariate_factorisation ( analyte_df , journal_df , formula ,
         return ( res_df )
 
 
+def associations ( M , W = None , bRanked = True ) :
+    ispanda = lambda P : 'pandas' in str(type(P)).lower()
+    if not ispanda( M ) :
+        print ( "FUNCTION ",'recast_alignments'," REQUIRES ", 'M'," TO BE A PANDAS DATAFRAME" )
+    bValid = False
+    if not W is None :
+        if not len(W.columns.values) == len(M.columns.values):
+            W = M
+        else:
+            bValid = True
+    else :
+        W = M
+    if bRanked :
+        from scipy.stats import rankdata
+        M = ( M.T.apply(lambda x:rankdata(x,'average')).T-0.5 )/len(M.columns)
+        W = ( W.T.apply(lambda x:rankdata(x,'average')).T-0.5 )/len(W.columns)
+    rho1 = M.T.apply( lambda x:np.sqrt( np.dot( x,x ) ) )
+    rho2 = rho1
+    if bValid :
+        rho2 = W.T.apply( lambda x:np.sqrt( np.dot( x,x ) ) )
+    R2  = pd.DataFrame( np.array([np.array([r]) for r in rho1.values])*[rho2.values] ,
+                        index = rho1.index, columns = rho2.index )
+    PQ  = pd.DataFrame( np.dot( M,W.T ), index = rho1.index, columns = rho2.index )
+    res = PQ/R2
+    return ( res )
+
+
 crop = lambda x,W:x[:,:W]
 def run_shape_alignment_regression( analyte_df , journal_df , formula ,
                           bVerbose = False , synonyms = None , blur_cutoff = 99.8 ,
@@ -475,40 +550,39 @@ def run_shape_alignment_regression( analyte_df , journal_df , formula ,
                           study_axii = None , owner_by = 'tesselation' ,
                           transform = crop ) :
 
-	NOTE__ = "Richard Tjörnhammars method that evolved as a synthesis of the work done together with Edward Tjörnhammar on the rpls method"
-	print ( 'WARNING: STILL UNDER DEVELOPMENT' )
-	print ( 'WARNING: DEFAULT IS TO CROP ALIGNED FACTORS!!')
+        print ( 'WARNING: STILL UNDER DEVELOPMENT' )
+        print ( 'WARNING: DEFAULT IS TO CROP ALIGNED FACTORS!!')
 
-	encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
+        encoding_df = interpret_problem ( analyte_df , journal_df , formula , bVerbose = bVerbose )
 
-	Q = encoding_df.T.apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).copy().values
-	P = analyte_df   .apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).copy().values
+        Q = encoding_df.T.apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).copy().values
+        P = analyte_df   .apply( lambda x:(rankdata(x,'average')-0.5)/len(x) ).copy().values
 
-	centroids = ifit.ShapeAlignment( P, Q ,
-				bReturnTransform = False ,
-				bShiftModel = True ,
-				bUnrestricted = True )
-	#
-	# FOR DIAGNOSTIC PURPOSES
-	centroids_df = pd.DataFrame ( centroids ,
-			index = encoding_df.columns ,
-			columns = encoding_df.index )
+        centroids = ifit.ShapeAlignment( P, Q ,
+                        bReturnTransform = False ,
+                        bShiftModel = True ,
+                        bUnrestricted = True )
+        #
+        # FOR DIAGNOSTIC PURPOSES
+        centroids_df = pd.DataFrame ( centroids ,
+                            index = encoding_df.columns ,
+                            columns = encoding_df.index )
 
-	xws = ifit.WeightsAndScoresOf( P )
-	yws = ifit.WeightsAndScoresOf( centroids )
+        xws = ifit.WeightsAndScoresOf( P )
+        yws = ifit.WeightsAndScoresOf( centroids )
 
-	W = np.min( [*np.shape(xws[0]),*np.shape(yws[0])] )
+        W = np.min( [*np.shape(xws[0]),*np.shape(yws[0])] )
 
-	quantx = transform( xws[0],W )
-	quanty = transform( yws[0],W )
-	scorex = transform( xws[1],W )
+        quantx = transform( xws[0],W )
+        quanty = transform( yws[0],W )
+        scorex = transform( xws[1],W )
 
-	res_df = calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
-			analyte_df = analyte_df.copy() , journal_df = journal_df.copy() ,
-			blur_cutoff = blur_cutoff , bVerbose = bVerbose,
-			exclude_labels_from_centroids = exclude_labels_from_centroids ,
-			study_axii = study_axii , owner_by = owner_by, synonyms=synonyms )
-	return ( res_df )
+        res_df = calculate_alignment_properties ( encoding_df , quantx, quanty, scorex,
+                    analyte_df = analyte_df.copy() , journal_df = journal_df.copy() ,
+                    blur_cutoff = blur_cutoff , bVerbose = bVerbose,
+                    exclude_labels_from_centroids = exclude_labels_from_centroids ,
+                    study_axii = study_axii , owner_by = owner_by, synonyms=synonyms )
+        return ( res_df )
 
 
 def add_foldchanges ( df, information_df , group='', fc_type=0 , foldchange_indentifier = 'FC,') :
@@ -639,16 +713,10 @@ def glm_test (  formula , df , jdf , distribution='Gaussian' ) :
 
 def t_test ( df , endogen = 'expression' , group = 'disease' ,
              pair_values = ('Sick','Healthy') , test_type = 'independent',
-             equal_var = False , alternative = 'greater' ,
-             bDeprecated = False ) :
+             equal_var = False , alternative = 'greater' ) :
 
-    if bDeprecated :
-        print ( 'WILL BE REMOVED IN FUTURE VERSIONS' ) 
-        group1 = df[df[group] == pair_values[0]][endogen].astype(float)
-        group2 = df[df[group] == pair_values[1]][endogen].astype(float)
-    else :
-        group1 = df.iloc[:,[n in pair_values[0] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
-        group2 = df.iloc[:,[n in pair_values[1] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
+    group1 = df.iloc[:,[n in pair_values[0] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
+    group2 = df.iloc[:,[n in pair_values[1] for n in df.loc[group,:].values] ].loc[endogen,:].astype(float)
 
     if test_type == 'independent' :
         pv = ttest_ind ( group1, group2 , equal_var = equal_var )
@@ -835,8 +903,6 @@ def quantify_groups_by_analyte_pvalues( analyte_df, grouping_file, delimiter='\t
             edf.loc[l] = q
     return ( edf.T )
 
-
-
 class APCA ( object ) :
     #
     # THIS CLASS PERFORMS A SPARSE PCA IF REQUESTED
@@ -909,12 +975,11 @@ class APCA ( object ) :
         self.components_ = self.V_
         return ( self.F_ )
 
-
 dimred = PCA()
 
 def quantify_groups ( analyte_df , journal_df , formula , grouping_file , synonyms = None ,
                       delimiter = '\t' , test_type = 'random' ,
-                      split_id = None , skip_line_char = '#' 
+                      split_id = None , skip_line_char = '#'
                     ) :
     statistical_formula = formula
     if not split_id is None :
@@ -1007,14 +1072,14 @@ def quantify_by_dictionary ( analyte_df , journal_df , formula , split_id=None,
                     metrics = [ c.split(',')[0] for c in loc_q.columns]
                     for metric in metrics:
                         statistic = qad.loc[ :, [c for c in qad.columns if metric in c and ',s' in c] ]
-                        group_analytes_pos_neg_ind_d[ metric + ',N_positive' ] = np.sum ( 
-                            [ 1 if p<tolerance and s>0 else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ] 
+                        group_analytes_pos_neg_ind_d[ metric + ',N_positive' ] = np.sum (
+                            [ 1 if p<tolerance and s>0 else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ]
                         )
-                        group_analytes_pos_neg_ind_d[ metric + ',N_negative' ] = np.sum ( 
-                            [ 1 if p<tolerance and s<0 else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ] 
+                        group_analytes_pos_neg_ind_d[ metric + ',N_negative' ] = np.sum (
+                            [ 1 if p<tolerance and s<0 else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ]
                         )
-                        group_analytes_pos_neg_ind_d[ metric + ',N_indetermined' ] = np.sum ( 
-                            [ 1 if p>tolerance else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ] 
+                        group_analytes_pos_neg_ind_d[ metric + ',N_indetermined' ] = np.sum (
+                            [ 1 if p>tolerance else 0 for (p,s) in zip(loc_q.loc[:,[metric+',p']].values,statistic.values) ]
                         )
                         group_analytes_pos_neg_ind_d[ metric + ',N_tot' ] = len(statistic)
 
@@ -1146,7 +1211,6 @@ def differential_analytes ( analyte_df , cols = [['a'],['b']] ):
     ddf = ddf.sort_values('Dist', ascending = False )
     return ( ddf )
 
-
 def add_kendalltau( analyte_results_df , journal_df , what='M' , sample_names = None, ForceSpearman=False ) :
     # ADD IN CONCORDANCE WITH KENDALL TAU
     if what in set(journal_df.index.values) :
@@ -1160,7 +1224,7 @@ def add_kendalltau( analyte_results_df , journal_df , what='M' , sample_names =
         else :
             patients = [ c for c in analyte_results_df.columns if '_' in c ]
         patients = [ p for p in sample_names if p in set(patients) &
-                     set( analyte_results_df.columns.values) & 
+                     set( analyte_results_df.columns.values) &
                      set( journal_df.loc[[what],:].T.dropna().T.columns.values ) ]
         for idx in analyte_results_df.index :
             y = journal_df.loc[ what,patients ].values
@@ -1219,9 +1283,11 @@ def calculate_rates( journal_df , inferred_df ,
 
     results_lookup = { 'TP':TP , 'TN':TN ,
                 'FN':FN ,'FP':FP ,
+                'causality'   : ( TP+TN ) / ( FP+FN ) ,
                 'sensitivity' : TP / ( TP+FN ) ,
                 'specificity' : TN / ( TN+FP ) ,
                 'precision'   : TP / ( TP+FP ) ,
+                'recall'      : TP / ( TP+FN ) ,
                 'accuracy'    : ( TP+TN ) / ( TP+TN+FP+FN ) ,
                 'negation'    : TN / ( TN+FN ) , # FNR
                 'FPR:'        : FP / ( FP+TN ) , # False positive rate
@@ -1256,6 +1322,18 @@ def isItPrime( N , M=None,p=None,lM05=None ) :
            return ( True )
        return ( isItPrime(N-1,M=M,p=p+1,lM05=lM05) )
 
+# FIRST APPEARENCE:
+# https://gist.github.com/richardtjornhammar/ef1719ab0dc683c69d5a864cb05c5a90
+def Fibonacci(n):
+    if n-2>0:
+        return ( Fibonacci(n-1)+Fibonacci(n-2) )
+    if n-1>0:
+        return ( Fibonacci(n-1) )
+    if n>0:
+       return ( n )
+
+def F_truth(i):
+    return ( Fibonacci(i)**2+Fibonacci(i+1)**2 == Fibonacci(2*i+1))
 
 if __name__ == '__main__' :
 
@@ -1270,3 +1348,11 @@ if __name__ == '__main__' :
     journal_df = prune_journal( pd.read_csv(journal_file,'\t', index_col=0 ) )
 
     print ( quantify_groups( analyte_df, journal_df, 'Group ~ Var + C(Cat) ', grouping_file ) )
+
+
+    import impetuous.quantification as impq
+    import sys
+    sys.setrecursionlimit(20000)
+    for i in range(6):
+        Fi = 2**(2**i)+1
+        print ("Fermats ",i,"th number = ", Fi, " and is it Prime?", impq.isItPrime(Fi) )
